@@ -32,10 +32,13 @@
 #include <libmissioncontrol/mc-profile.h>
 
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-debug.h>
 
 #include "empathy-account-widget-jabber.h"
 #include "empathy-ui-utils.h"
 
+#define DEBUG_DOMAIN "AccountWidgetJabber"
+
 #define PORT_WITHOUT_SSL 5222
 #define PORT_WITH_SSL 5223
 
@@ -51,23 +54,10 @@ typedef struct {
 	GtkWidget *spinbutton_port;
 	GtkWidget *spinbutton_priority;
 	GtkWidget *checkbutton_ssl;
+	GtkWidget *checkbutton_ignore_ssl_errors;
+	GtkWidget *checkbutton_encryption;
 } EmpathyAccountWidgetJabber;
 
-static gboolean account_widget_jabber_entry_focus_cb           (GtkWidget                 *widget,
-								GdkEventFocus             *event,
-								EmpathyAccountWidgetJabber *settings);
-static void     account_widget_jabber_entry_changed_cb         (GtkWidget                 *widget,
-								EmpathyAccountWidgetJabber *settings);
-static void     account_widget_jabber_checkbutton_toggled_cb   (GtkWidget                 *widget,
-								EmpathyAccountWidgetJabber *settings);
-static void     account_widget_jabber_value_changed_cb         (GtkWidget                 *spinbutton,
-								EmpathyAccountWidgetJabber *settings);
-static void     account_widget_jabber_button_forget_clicked_cb (GtkWidget                 *button,
-								EmpathyAccountWidgetJabber *settings);
-static void     account_widget_jabber_destroy_cb               (GtkWidget                 *widget,
-								EmpathyAccountWidgetJabber *settings);
-static void     account_widget_jabber_setup                    (EmpathyAccountWidgetJabber *settings);
-
 static gboolean
 account_widget_jabber_entry_focus_cb (GtkWidget                 *widget,
 				      GdkEventFocus             *event,
@@ -95,10 +85,13 @@ account_widget_jabber_entry_focus_cb (GtkWidget                 *widget,
 	if (G_STR_EMPTY (str)) {
 		gchar *value = NULL;
 
+		mc_account_unset_param (settings->account, param);
 		mc_account_get_param_string (settings->account, param, &value);
+		empathy_debug (DEBUG_DOMAIN, "Unset %s and restore to %s", param, value);
 		gtk_entry_set_text (GTK_ENTRY (widget), value ? value : "");
 		g_free (value);
 	} else {
+		empathy_debug (DEBUG_DOMAIN, "Setting %s to %s", param, str);
 		mc_account_set_param_string (settings->account, param, str);
 	}
 
@@ -121,26 +114,50 @@ static void
 account_widget_jabber_checkbutton_toggled_cb (GtkWidget                 *widget,
 					      EmpathyAccountWidgetJabber *settings)
 {
+	gboolean     value;
+	gboolean     default_value;
+	const gchar *param;
+
+	value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
 	if (widget == settings->checkbutton_ssl) {
-		gint     port = 0;
-		gboolean old_ssl;
+		gint port = 0;
 
 		mc_account_get_param_int (settings->account, "port", &port);
-		old_ssl = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
-		if (old_ssl) {
-			if (port == PORT_WITHOUT_SSL) {
+		if (value) {
+			if (port == PORT_WITHOUT_SSL || port == 0) {
 				port = PORT_WITH_SSL;
 			}
 		} else {
-			if (port == PORT_WITH_SSL) {
+			if (port == PORT_WITH_SSL || port == 0) {
 				port = PORT_WITHOUT_SSL;
 			}
 		}
-		
-		mc_account_set_param_int (settings->account, "port", port);
-		mc_account_set_param_boolean (settings->account, "old-ssl", old_ssl);
+
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (settings->spinbutton_port), port);
+		param = "old-ssl";
+	}
+	else if (widget == settings->checkbutton_ignore_ssl_errors) {
+		param = "ignore-ssl-errors";
+	}
+	else if (widget == settings->checkbutton_encryption) {
+		param = "require-encryption";
+	} else {
+		return;
+	}
+
+	/* FIXME: This is ugly! checkbox don't have a "not-set" value so we
+	 * always unset the param and set the value if different from the
+	 * default value. */
+	mc_account_unset_param (settings->account, param);
+	mc_account_get_param_boolean (settings->account, param, &default_value);
+
+	if (default_value != value) {
+		empathy_debug (DEBUG_DOMAIN, "Setting %s to %d", param, value);
+		mc_account_set_param_boolean (settings->account, param, value);
+	} else {
+		empathy_debug (DEBUG_DOMAIN, "Unset %s", param, value);
 	}
 }
 
@@ -148,15 +165,30 @@ static void
 account_widget_jabber_value_changed_cb (GtkWidget                 *spinbutton,
 					EmpathyAccountWidgetJabber *settings)
 {
-	gdouble value;
+	gdouble      value;
+	const gchar *param;
 
 	value = gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbutton));
 
 	if (spinbutton == settings->spinbutton_port) {
-		mc_account_set_param_int (settings->account, "port", (gint) value);
+		param = "port";
 	}
 	else if (spinbutton == settings->spinbutton_priority) {
-		mc_account_set_param_int (settings->account, "priority", (gint) value);
+		param = "priority";
+	} else {
+		return;
+	}
+
+	if (value != 0) {
+		empathy_debug (DEBUG_DOMAIN, "Setting %s to %d", param, (gint) value);
+		mc_account_set_param_int (settings->account, param, (gint) value);
+	} else {
+		gint val;
+
+		mc_account_unset_param (settings->account, param);
+		mc_account_get_param_int (settings->account, param, &val);
+		empathy_debug (DEBUG_DOMAIN, "Unset %s and restore to %d", param, val);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton), val);
 	}
 }
 
@@ -164,7 +196,8 @@ static void
 account_widget_jabber_button_forget_clicked_cb (GtkWidget                 *button,
 						EmpathyAccountWidgetJabber *settings)
 {
-	mc_account_set_param_string (settings->account, "password", "");
+	empathy_debug (DEBUG_DOMAIN, "Unset password");
+	mc_account_unset_param (settings->account, "password");
 	gtk_entry_set_text (GTK_ENTRY (settings->entry_password), "");
 }
 
@@ -186,6 +219,8 @@ account_widget_jabber_setup (EmpathyAccountWidgetJabber *settings)
 	gchar    *server = NULL;
 	gchar    *password = NULL;
 	gboolean  old_ssl = FALSE;
+	gboolean  ignore_ssl_errors = FALSE;
+	gboolean  encryption = FALSE;
 
 	mc_account_get_param_int (settings->account, "port", &port);
 	mc_account_get_param_int (settings->account, "priority", &priority);
@@ -194,6 +229,8 @@ account_widget_jabber_setup (EmpathyAccountWidgetJabber *settings)
 	mc_account_get_param_string (settings->account, "server", &server);
 	mc_account_get_param_string (settings->account, "password", &password);
 	mc_account_get_param_boolean (settings->account, "old-ssl", &old_ssl);
+	mc_account_get_param_boolean (settings->account, "ignore-ssl-errors", &ignore_ssl_errors);
+	mc_account_get_param_boolean (settings->account, "require-encryption", &encryption);
 
 	if (!id) {
 		McProfile   *profile;
@@ -207,13 +244,15 @@ account_widget_jabber_setup (EmpathyAccountWidgetJabber *settings)
 		g_object_unref (profile);
 	}
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (settings->checkbutton_ssl), old_ssl);
-	gtk_entry_set_text (GTK_ENTRY (settings->entry_id), id ? id : "");
-	gtk_entry_set_text (GTK_ENTRY (settings->entry_password), password ? password : "");
-	gtk_entry_set_text (GTK_ENTRY (settings->entry_resource), resource ? resource : "");
-	gtk_entry_set_text (GTK_ENTRY (settings->entry_server), server ? server : "");
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (settings->spinbutton_port), port);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (settings->spinbutton_priority), priority);
+	gtk_entry_set_text (GTK_ENTRY (settings->entry_id), id ? id : "");
+	gtk_entry_set_text (GTK_ENTRY (settings->entry_resource), resource ? resource : "");
+	gtk_entry_set_text (GTK_ENTRY (settings->entry_server), server ? server : "");
+	gtk_entry_set_text (GTK_ENTRY (settings->entry_password), password ? password : "");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (settings->checkbutton_ssl), old_ssl);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (settings->checkbutton_ignore_ssl_errors), ignore_ssl_errors);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (settings->checkbutton_encryption), encryption);
 
 	gtk_widget_set_sensitive (settings->button_forget, !G_STR_EMPTY (password));
 
@@ -228,7 +267,6 @@ empathy_account_widget_jabber_new (McAccount *account)
 {
 	EmpathyAccountWidgetJabber *settings;
 	GladeXML                  *glade;
-	GtkSizeGroup              *size_group;
 	GtkWidget                 *label_id, *label_password;
 	GtkWidget                 *label_server, *label_resource;
 	GtkWidget                 *label_port, *label_priority; 
@@ -254,6 +292,8 @@ empathy_account_widget_jabber_new (McAccount *account)
 				       "spinbutton_port", &settings->spinbutton_port,
 				       "spinbutton_priority", &settings->spinbutton_priority,
 				       "checkbutton_ssl", &settings->checkbutton_ssl,
+				       "checkbutton_ignore_ssl_errors", &settings->checkbutton_ignore_ssl_errors,
+				       "checkbutton_encryption", &settings->checkbutton_encryption,
 				       NULL);
 
 	account_widget_jabber_setup (settings);
@@ -270,25 +310,12 @@ empathy_account_widget_jabber_new (McAccount *account)
 			      "entry_resource", "focus-out-event", account_widget_jabber_entry_focus_cb,
 			      "entry_server", "focus-out-event", account_widget_jabber_entry_focus_cb,
 			      "checkbutton_ssl", "toggled", account_widget_jabber_checkbutton_toggled_cb,
+			      "checkbutton_ignore_ssl_errors", "toggled", account_widget_jabber_checkbutton_toggled_cb,
+			      "checkbutton_encryption", "toggled", account_widget_jabber_checkbutton_toggled_cb,
 			      NULL);
 
 	g_object_unref (glade);
 
-	/* Set up remaining widgets */
-	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-
-	gtk_size_group_add_widget (size_group, label_id);
-	gtk_size_group_add_widget (size_group, label_password);
-	gtk_size_group_add_widget (size_group, label_resource);
-	gtk_size_group_add_widget (size_group, label_server);
-	gtk_size_group_add_widget (size_group, label_port);
-	gtk_size_group_add_widget (size_group, label_priority);
-
-	g_object_unref (size_group);
-
-	gtk_editable_select_region (GTK_EDITABLE (settings->entry_id), 0, -1);
-	gtk_widget_grab_focus (settings->entry_id);
-
 	gtk_widget_show (settings->vbox_settings);
 
 	return settings->vbox_settings;
