@@ -62,7 +62,7 @@ empathy_chandler_class_init (EmpathyChandlerClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_OBJECT,
+			      _empathy_marshal_VOID__OBJECT_OBJECT,
 			      G_TYPE_NONE,
 			      2, TELEPATHY_CONN_TYPE, TELEPATHY_CHAN_TYPE);
 }
