@@ -208,11 +208,15 @@ empathy_xml_validate (xmlDoc      *doc,
 	xmlDtd       *dtd;
 	gboolean      ret;
 
-	path = g_build_filename (DATADIR, "empathy", dtd_filename, NULL);
+	path = g_build_filename (UNINSTALLED_DTD_DIR, dtd_filename, NULL);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
+		g_free (path);
+		path = g_build_filename (DATADIR, "empathy", dtd_filename, NULL);
+	}
+	empathy_debug (DEBUG_DOMAIN, "Loading dtd file %s", path);
 
 	/* The list of valid chars is taken from libxml. */
 	escaped = xmlURIEscapeStr (path, ":@&=+$,/?;");
-
 	g_free (path);
 
 	memset (&cvp, 0, sizeof (cvp));
@@ -393,61 +397,36 @@ empathy_inspect_handle (McAccount *account,
 }
 
 void
-empathy_call_contact (EmpathyContact *contact)
+empathy_call_with_contact (EmpathyContact  *contact)
 {
 #ifdef HAVE_VOIP
 	MissionControl *mc;
-	McAccount      *account;
-	TpConn         *tp_conn;
-	gchar          *object_path;
-	const gchar    *bus_name;
-	TpChan         *new_chan;
-	EmpathyTpGroup *group;
-	GError         *error = NULL;
-
-	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
-
-	/* StreamedMedia channels must have handle=0 and handle_type=none.
-	 * To call a contact we have to add him in the group interface of the
-	 * channel. MissionControl will detect the channel creation and 
-	 * dispatch it to the VoIP chandler automatically. */
 
 	mc = empathy_mission_control_new ();
-	account = empathy_contact_get_account (contact);
-	tp_conn = mission_control_get_connection (mc, account, NULL);
-	/* FIXME: Should be async */
-	if (!tp_conn_request_channel (DBUS_G_PROXY (tp_conn),
-				      TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-				      TP_HANDLE_TYPE_NONE,
-				      0,
-				      FALSE,
-				      &object_path,
-				      &error)) {
-		empathy_debug (DEBUG_DOMAIN, 
-			      "Couldn't request channel: %s",
-			      error ? error->message : "No error given");
-		g_clear_error (&error);
-		g_object_unref (mc);
-		g_object_unref (tp_conn);
-		return;
-	}
+	mission_control_request_channel (mc,
+					 empathy_contact_get_account (contact),
+					 TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+					 empathy_contact_get_handle (contact),
+					 TP_HANDLE_TYPE_CONTACT,
+					 NULL, NULL);
+	g_object_unref (mc);
+#endif
+}
 
-	bus_name = dbus_g_proxy_get_bus_name (DBUS_G_PROXY (tp_conn));
-	new_chan = tp_chan_new (tp_get_bus (),
-				bus_name,
-				object_path,
-				TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-				TP_HANDLE_TYPE_NONE,
-				0);
+void
+empathy_call_with_contact_id (McAccount *account, const gchar *contact_id)
+{
+#ifdef HAVE_VOIP
+	MissionControl *mc;
 
-	group = empathy_tp_group_new (account, new_chan);
-	empathy_tp_group_add_member (group, contact, "");
-
-	g_object_unref (group);
+	mc = empathy_mission_control_new ();
+	mission_control_request_channel_with_string_handle (mc,
+							    account,
+							    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+							    contact_id,
+							    TP_HANDLE_TYPE_CONTACT,
+							    NULL, NULL);
 	g_object_unref (mc);
-	g_object_unref (tp_conn);
-	g_object_unref (new_chan);
-	g_free (object_path);
 #endif
 }
 
@@ -481,3 +460,73 @@ empathy_chat_with_contact_id (McAccount *account, const gchar *contact_id)
 	g_object_unref (mc);
 }
 
+const gchar *
+empathy_presence_get_default_message (McPresence presence)
+{
+	switch (presence) {
+	case MC_PRESENCE_AVAILABLE:
+		return _("Available");
+	case MC_PRESENCE_DO_NOT_DISTURB:
+		return _("Busy");
+	case MC_PRESENCE_AWAY:
+	case MC_PRESENCE_EXTENDED_AWAY:
+		return _("Away");
+	case MC_PRESENCE_HIDDEN:
+		return _("Hidden");
+	case MC_PRESENCE_OFFLINE:
+	case MC_PRESENCE_UNSET:
+		return _("Offline");
+	default:
+		g_assert_not_reached ();
+	}
+
+	return NULL;
+}
+
+const gchar *
+empathy_presence_to_str (McPresence presence)
+{
+	switch (presence) {
+	case MC_PRESENCE_AVAILABLE:
+		return "available";
+	case MC_PRESENCE_DO_NOT_DISTURB:
+		return "busy";
+	case MC_PRESENCE_AWAY:
+		return "away";
+	case MC_PRESENCE_EXTENDED_AWAY:
+		return "ext_away";
+	case MC_PRESENCE_HIDDEN:
+		return "hidden";
+	case MC_PRESENCE_OFFLINE:
+		return "offline";
+	case MC_PRESENCE_UNSET:
+		return "unset";
+	default:
+		g_assert_not_reached ();
+	}
+
+	return NULL;
+}
+
+McPresence
+empathy_presence_from_str (const gchar *str)
+{
+	if (strcmp (str, "available") == 0) {
+		return MC_PRESENCE_AVAILABLE;
+	} else if ((strcmp (str, "dnd") == 0) || (strcmp (str, "busy") == 0)) {
+		return MC_PRESENCE_DO_NOT_DISTURB;
+	} else if ((strcmp (str, "away") == 0) || (strcmp (str, "brb") == 0)) {
+		return MC_PRESENCE_AWAY;
+	} else if ((strcmp (str, "xa") == 0) || (strcmp (str, "ext_away") == 0)) {
+		return MC_PRESENCE_EXTENDED_AWAY;
+	} else if (strcmp (str, "hidden") == 0) {
+		return MC_PRESENCE_HIDDEN;
+	} else if (strcmp (str, "offline") == 0) {
+		return MC_PRESENCE_OFFLINE;
+	} else if (strcmp (str, "unset") == 0) {
+		return MC_PRESENCE_UNSET;
+	}
+
+	return MC_PRESENCE_AVAILABLE;
+}
+
