@@ -45,7 +45,7 @@ main (int argc, char *argv[])
 	gtk_init (&argc, &argv);
 
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
-					   DATADIR G_DIR_SEPARATOR_S "empathy");
+					   PKGDATADIR G_DIR_SEPARATOR_S "icons");
 	dialog = empathy_accounts_dialog_show (NULL);
 
 	g_signal_connect (dialog, "destroy",
